/*
 * Copyright 2010, 2012 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Font.h"
#include "Path.h"
#include <fstream>

Font::Font(const std::string& file)
{
	Surface fontSurface("fonts/" + file + ".png");
	fontHeight = fontSurface.height();
	
	std::ifstream fontFile((dataPath() + "fonts" + (file + ".font")).c_str());
	
	int x = 0;
	char c;
	int charWidth;
	
	fontFile >> charWidth;
	
	if (fontFile.good())
	{
		charMap.insert(std::make_pair(' ', fontSurface.subSurface(0, 0, charWidth, fontHeight)));
		x += charWidth;
	}

	while (fontFile >> c >> charWidth && (x + charWidth) <= fontSurface.width())
	{
		charMap.insert(std::make_pair(c, fontSurface.subSurface(x, 0, charWidth, fontHeight)));
		x += charWidth;
	}
}



Surface Font::createTextSurface(const std::string& str, Color c)
{
	if (c.r == 0xFF && c.g == 0xFF && c.b == 0xFF)
	{
		c.r = 0xFE;
		c.g = 0xFE;
		c.b = 0xFE;
	}

	int width = 0;
	for (std::string::const_iterator it = str.begin(); it != str.end(); ++it)
	{
		std::map<char, Surface>::iterator mit = charMap.find(std::tolower(*it));
		if (mit != charMap.end())
		{
			width += mit->second.width();
		}
	}
	
	Surface textSurface(width, fontHeight);
	
	width = 0;
	for (std::string::const_iterator it = str.begin(); it != str.end(); ++it)
	{
		std::map<char, Surface>::iterator mit = charMap.find(std::tolower(*it));
		if (mit != charMap.end())
		{
			mit->second.draw(textSurface, width, 0);
			width += mit->second.width();
		}
	}
	textSurface.setColorKey(0x00, 0x00, 0x00);
	Surface textSurface2(width, fontHeight);
	textSurface2.setColorKey(0xFF, 0xFF, 0xFF);
	textSurface2.fill(c.r, c.g, c.b);
	textSurface.draw(textSurface2, 0, 0);
	
	return textSurface2;
}
