/*
 * Copyright 2010 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MAP_H
#define MAP_H

const int TILE_SIZE = 32;

const int MAX_MAP_X = 10;
const int MAX_MAP_Y = 7;

// Directions
const int LEFT  = 0x01;
const int RIGHT = 0x02;
const int UP    = 0x04;
const int DOWN  = 0x08;

// Road connections
const int ROAD_HORIZONTAL = LEFT  | RIGHT;
const int ROAD_VERTICAL   = UP    | DOWN;
const int ROAD_LEFT_UP    = LEFT  | UP;
const int ROAD_LEFT_DOWN  = LEFT  | DOWN;
const int ROAD_RIGHT_UP   = RIGHT | UP;
const int ROAD_RIGHT_DOWN = RIGHT | DOWN;

// A map should be seen as a real map. 
// It is just a description of a track;
class Map
{
public:
	void generate();
	void zeroFill();
	int getRoadCount() const; // gives #tiles occupied by road
	const int* operator[](int y) const;
	void print() const;
private:
	void makeTrack();
	void makeConnection(int lowerX, int upperX, int lowerY, int upperY, int connectionX1, int connectionY1, int connectionDir1, int connectionX2, int connectionY2, int connectionDir2);
	void center();
	int map[MAX_MAP_Y][MAX_MAP_X];
};

#endif
