/*
 * Copyright 2011, 2013 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MENU_H
#define MENU_H

#include "RenderObject.h"
#include <vector>

class GameState;

extern const Color STD_INACTIVE_COLOR;
extern const Color STD_ACTIVE_COLOR;
extern const Color STD_ACTIVE_COLOR2;

class Menu
{
public:
	Menu();
	void addItem(const std::string& label, int* value, int min, int max, const char* firstStrValue = 0, ...);
	void addItem(const std::string& label, bool* fireAction);
	void addItem(const std::string& label, bool* fireAction, int* value, int min, int max, const char* firstStrValue = 0, ...);
	void addItem(const std::string& label, bool* fireAction, int* value, const std::vector<std::string>& strValues);
	void addRenderObjects(GameState* gs, int x, int y, int dx, int dy);
	bool onKeyDown(SDLKey key);
	
	void next();
	void prev();
	void action();
	void inc();
	void dec();
	
	void notifyValueUpdate(int* value);
	void selectLastItem();
	bool activeItemValue(int* value) const;
	void setActiveColor(const Color& color);
	void setLabel(int i, const std::string& label);
	
	void reset(GameState* gs);
	
private:
	void updateItem(std::size_t i);
	struct Item
	{
		std::string label;
		bool* fireAction;
		int* value;
		int min;
		int max;
		bool wrap;
		std::vector<std::string> strValues;
		RenderObject renderObject;
	};
	std::vector<Item> items;
	std::size_t selectedItem;
	Color activeColor;
	Color inactiveColor;
};

#endif
