/*
 * Copyright 2010, 2011, 2012, 2013 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "RacePreparationMenu.h"
#include "StateManager.h"
#include <fstream>

const int MAX_PLAYERS = 8;
const int MIN_PLAYERS = 1;
const int MAX_LAPS = 9;
const int MIN_LAPS = 1;
const int MAX_TRACKS = 100;
const int MIN_TRACKS = 1;

static const Color inactiveColor = {0x00, 0x00, 0x00};
static const Color activeColor = {0x20, 0xFF, 0x20};

RacePreparationMenu::RacePreparationMenu(bool tournament)
:	humanPlayers(1),
	computerPlayers(3),
	laps(3),
	tracks(tournament ? 4 : 1),
	tournament(tournament),
	collisions(1),
	difficulty(2),
	moveToNext(false),
	actionNext(false),
	path(configPath() + (tournament ? "/tournament" : "/single"))
{	
	load();
	background.fill(120, 124, 224);
	menu.addItem("Human players", &moveToNext, &humanPlayers, 0, 8);
	menu.addItem("Computer players", &moveToNext, &computerPlayers, 0, 8);
	menu.addItem("Laps", &moveToNext, &laps, 1, 9);
	if (tournament)
	{
		menu.addItem("Tracks", &moveToNext, &tracks, 1, 100);
	}
	menu.addItem("Car collisions", &moveToNext, &collisions, 0, 1, "off", "on");
	menu.addItem("Computer difficulty", &moveToNext, &difficulty, 0, 4, "very easy", "easy", "normal", "hard", "very hard");
	menu.addItem("Next", &actionNext);
	
	menu.addRenderObjects(this, 10, 10, 10, 10);
}

RacePreparationMenu::~RacePreparationMenu()
{
	save();
}

void RacePreparationMenu::onKeyDown(SDLKey key)
{
	if (menu.onKeyDown(key))
	{
		if (moveToNext)
		{
			menu.selectLastItem();
			moveToNext = false;
		}
		else if (actionNext)
		{
			StateManager::getInstance().enterCarSelectionState(humanPlayers, computerPlayers, laps, tracks, tournament, collisions, difficulty);
		}
		else
		{
			if (humanPlayers + computerPlayers > 8)
			{
				if (menu.activeItemValue(&humanPlayers))
				{
					computerPlayers = 8 - humanPlayers;
					menu.notifyValueUpdate(&computerPlayers);
				}
				else
				{
					humanPlayers = 8 - computerPlayers;
					menu.notifyValueUpdate(&humanPlayers);
				}
			}
			else if (humanPlayers + computerPlayers <= 0)
			{
				if (menu.activeItemValue(&humanPlayers))
				{
					computerPlayers = 1;
					menu.notifyValueUpdate(&computerPlayers);
				}
				else
				{
					humanPlayers = 1;
					menu.notifyValueUpdate(&humanPlayers);
				}
			}
		}
	}
	else if (key == SDLK_ESCAPE)
	{
		StateManager::getInstance().enterMainMenuState();
	}
}

void RacePreparationMenu::save() const
{
	std::ofstream out(path.c_str());
	out << humanPlayers << ' ';
	out << computerPlayers << ' ';
	out << laps << ' ';
	if (tournament)
	{
		out << tracks << ' ';
	}
	out << collisions << ' ';
	out << difficulty;
}

void RacePreparationMenu::load()
{
	std::ifstream in(path.c_str());
	if (in.is_open())
	{
		in >> humanPlayers;
		in >> computerPlayers;
		in >> laps;
		if (tournament)
		{
			in >> tracks;
		}
		in >> collisions;
		in >> difficulty;
		
		// give approppriate values!
		humanPlayers = std::min(std::max(humanPlayers, 0), 8);
		computerPlayers = std::min(std::max(computerPlayers, 0), 8);
		while (humanPlayers + computerPlayers > 8)
		{
			--computerPlayers;
		}
		if (humanPlayers + computerPlayers == 0)
		{
			humanPlayers = 0;
		}
		laps = std::min(std::max(laps, 1), 100);
		tracks = std::min(std::max(tracks, 1), 8);
		collisions = std::min(std::max(collisions, 0), 1);
		difficulty = std::min(std::max(difficulty, 0), 4);
	}
}
