/*
 * Copyright 2010 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef RACESETUP_H
#define RACESETUP_H

#include <vector>

class Driver;

class RaceSetup
{
public:
	RaceSetup(const std::vector<Driver*>& drivers, const std::vector<int>& playerCars, int laps, int tracks, bool tournament, bool carCollisions);
	bool isTournament() const;
	bool carCollisionsEnabled() const;
	int numberOfLaps() const;
	int numberOfPlayers() const;
	void setPlayerPoints(int player, int points);
	int getPlayerPoints(int player) const;
	void setPlayerTime(int player, int time);
	int getPlayerTime(int player) const;
	int getPlayerCar(int player) const;
	int getPlayerPosition(int player) const;
	void givePoints();
	std::vector<Driver*> getDrivers() const;
	
	void nextTrack();
	int getTracks() const;
	int getCurrentTrack() const;
	
	void sortPlayersByPoints();
	void sortPlayersByTime();
	void reversePlayerOrder();
	
private:
	struct Player
	{
		int car;
		Driver* driver;
		int points;
		int time;
		int pos;
	};
	static bool pointsCmp(const Player& p1, const Player& p2);
	static bool timeCmp(const Player& p1, const Player& p2);
	
	std::vector<Player> players;
	bool tournament;
	bool carCollisions;
	int laps;
	int tracks;
	int currentTrack;
};

#endif
