/*
 * Copyright 2010, 2011, 2012 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef RACESTATE_H
#define RACESTATE_H

#include "GameState.h"
#include "Track.h"
#include "RenderObject.h"
#include "RaceSetup.h"

class Driver;
class Map;

class RaceState : public GameState
{
public:
	RaceState(const Map& map, const RaceSetup& raceSetup);
	virtual void update();
	void updateTimeState();
	virtual int sleepTime();
	virtual void onKeyDown(SDLKey key);
	
private:
	void updateLapRO(int i);
	void updatePosRO(int i);
	void updateTimeRO();
	
	struct Car
	{
		RenderObject carRO;
		RenderObject posRO;
		RenderObject timeRO;
		RenderObject lapRO;
		int pos;
		int lap;
		bool finished;
	};
	RaceSetup raceSetup;

	std::vector<Car> cars;
	
	//RenderObject renderCars[8];
	Track track;
	Uint32 gameTime;
	Uint32 raceTime;
	
	struct CarProgress
	{
		int carId;
		int progress;
		bool operator<(const RaceState::CarProgress& cp) const;
	};
	CarProgress carProgress[8];
	Surface smallNumbers[10];
	Surface bigNumbers[10];
	Uint32 timeNumbers[6];
	RenderObject timeRO[6];
	RenderObject readyRO;
	RenderObject setRO;
	RenderObject goRO;
	bool endRace;
	bool returnFromMenu;
	
	enum
	{
		TIME_STATE_READY,
		TIME_STATE_SET,
		TIME_STATE_GO,
		TIME_STATE_TIME
	} timeState;

};

#endif
