/*
 * Copyright 2010, 2011, 2012, 2013 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef STATEMANAGER_H
#define STATEMANAGER_H

#include <stack>
#include <vector>
#include <list>
#include "SDL.h"
#include "Rect.h"
#include "Surface.h"

const int GAME_WIDTH = 320;
const int GAME_HEIGHT = 240;

class GameState;
class Driver;
class RaceSetup;
class InputDevice;

class StateManager
{
public:
	void cleanUp();
	void run();
	void addUpdateRect(Rect& rect);
	static StateManager& getInstance();
	
	void enterQuitState();
	void enterMainMenuState();
	void enterRacePreparationMenuState(bool tournament);
	void enterCarSelectionState(int humanPlayers, int computerPlayers, int laps, int tracks, bool tournament, bool carCollisions, int difficulty);
	void enterRaceState(const RaceSetup& raceSetup);
	void enterInRaceMenuState(bool& endRace);
	void enterResultState(const RaceSetup& raceSetup);
	void enterControllerSettings();
	void enterPrevState();
	
	void updateWholeScreen();
	
	Driver* getDriver(int index);
	InputDevice* getDevice(int index);
	std::vector<std::string> getDeviceNames() const;
	void setDriver(int i, Driver* driver);
	
private:
	StateManager();
	void resize(int width, int height);
	void handleEvent(const SDL_Event& event);
	void updateVideoMode();
	void updateScreen();
	void setNewState(GameState* state);
	void screenToBackgound();
	void savePrevState();
	void toggleFullscreen();
	
	GameState* currentState;
	Surface screenBuffer;
	bool stateChanged;
	
	Surface prevStatebackground;
	GameState* prevState;
	
	// graphics stuff
	
	int screenZoom;
	int screenPaddingX;
	int screenPaddingY;
	bool fullscreen;
	// the fullscreen variable is used as an 
	// index to get screenWidth and screenHeight.
	int screenWidth[2];
	int screenHeight[2];
	
	std::vector<SDL_Rect> updateSDLRects;
	std::list<Rect> updateRects;
	std::vector<InputDevice*> devices;
	std::vector<Driver*> drivers;
};

#endif
