/*
 * Copyright 2010, 2011 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SURFACE_H
#define SURFACE_H

#include "SDL.h"
#include <string>

enum FontType
{
	SMALL_FONT,
	BIG_FONT
};

typedef SDL_Color Color;

class Surface
{
public:
	Surface();
	Surface(int width, int height);
	Surface(const std::string& filename);
	Surface(const std::string& str, FontType font, const Color& c);
	Surface(const Surface& s);
	~Surface();
	Surface& operator=(const Surface& s);

	Surface subSurface(int x, int y, int w, int h) const;
	void draw(Surface& dest, int x, int y) const;
	void draw(Surface& dest, int x, int y, double angle) const;
	void drawToScreen();
	int width() const;
	int height() const;
	void setColorKey(Uint8 r, Uint8 g, Uint8 b);
	void disableColorKey();
	void fill(Uint8 r, Uint8 g, Uint8 b, SDL_Rect* rect = 0);
	void setColor(int x, int y, Uint32 color);
	Uint32 getColor(int x, int y) const;
private:
	void validateClipping();

	mutable SDL_Surface* surface;
	mutable SDL_Rect clipping;
	Color colorKey;
	friend void zoom_blit(int,int,int,const Surface&,SDL_Rect*);
};

// converts integers to text.
std::string int2string(int i);
std::string time2string(int t);

#endif
